@@ -15,6 +15,7 @@ import (
 	"text/tabwriter"
 	"time"
 
+	"github.com/apex/log"
 	"github.com/blacktop/go-macho"
 	"github.com/blacktop/go-macho/types"
 	"github.com/blacktop/ipsw/internal/commands/dsc"
@@ -569,9 +570,15 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 		case "KernelCache":
 			i.Payload.BinaryImages[idx].Name = "kernelcache"
 			total--
+		case "KernelTextExec":
+			i.Payload.BinaryImages[idx].Name = "kernelcache (__TEXT_EXEC)"
+			total--
 		case "SharedCache":
 			i.Payload.BinaryImages[idx].Name = "dyld_shared_cache"
 			total--
+		case "SharedCacheLibrary":
+			i.Payload.BinaryImages[idx].Name = "dyld_shared_cache (library)"
+			total--
 		}
 	}
 
@@ -582,7 +589,7 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 		}
 		for idx, img := range i.Payload.BinaryImages {
 			var slide uint64
-			if strings.EqualFold(img.UUID, m.UUID().UUID.String()) {
+			if m.UUID() != nil && strings.EqualFold(img.UUID, m.UUID().UUID.String()) {
 				i.Payload.BinaryImages[idx].Path = path
 				i.Payload.BinaryImages[idx].Name = path
 				slide = i.Payload.BinaryImages[idx].Base - m.GetBaseAddress()
@@ -631,6 +638,9 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 				if len(i.Payload.BinaryImages[frame.ImageIndex].Name) > 0 {
 					i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName = i.Payload.BinaryImages[frame.ImageIndex].Name
 				}
+				if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName == "absolute" {
+					continue
+				}
 				i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset += i.Payload.BinaryImages[frame.ImageIndex].Base
 				if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName == "dyld_shared_cache" {
 					// lookup symbol in DSC dylib
@@ -666,6 +676,13 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 								i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = fn.Name + delta
 							}
 						}
+					} else {
+						log.WithFields(log.Fields{
+							"proc":   fmt.Sprintf("%s [%d]", proc.Name, pid),
+							"thread": tid,
+							"frame":  idx,
+							"img":    i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName,
+						}).Debugf("failed to find function for offset %#x", i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset)
 					}
 				}
 			}
@@ -787,6 +804,12 @@ func (i *Ips) String(verbose bool) string {
 			}
 			out += "\n"
 		}
+		if len(i.Payload.Notes) > 0 {
+			out += colorField("NOTES") + ":\n"
+			for _, n := range i.Payload.Notes {
+				out += fmt.Sprintf("    - %s\n", n)
+			}
+		}
 	case "Crash", "309":
 		out = fmt.Sprintf("[%s] - %s\n\n", colorTime(i.Header.Timestamp.Format("02Jan2006 15:04:05")), colorError(i.Header.BugTypeDesc))
 		out += fmt.Sprintf(
